# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import deprecated
from ....core import same_doc_as

from ..models import ErrorEntity
from ..models import FulfillCodeRequest
from ..models import FulfillmentHistoryPagingSlicedResult
from ..models import FulfillmentItem
from ..models import FulfillmentPagingSlicedResult
from ..models import FulfillmentRequest
from ..models import FulfillmentResult
from ..models import FulfillmentV2Request
from ..models import FulfillmentV2Result
from ..models import FulfillmentV3Request
from ..models import PreCheckFulfillmentRequest
from ..models import RevokeFulfillmentV2Result
from ..models import RewardsRequest

from ..operations.fulfillment import BulkFulfillItemsV3
from ..operations.fulfillment import FulfillItem
from ..operations.fulfillment import FulfillItems
from ..operations.fulfillment import FulfillItemsV3
from ..operations.fulfillment import FulfillRewards
from ..operations.fulfillment import FulfillRewardsV2
from ..operations.fulfillment import PreCheckFulfillItem
from ..operations.fulfillment import PublicRedeemCode
from ..operations.fulfillment import QueryFulfillmentHistories
from ..operations.fulfillment import QueryFulfillmentHistoriesStatusEnum
from ..operations.fulfillment import QueryFulfillments
from ..operations.fulfillment import QueryFulfillmentsStateEnum
from ..operations.fulfillment import RedeemCode
from ..operations.fulfillment import RetryFulfillItems
from ..operations.fulfillment import RetryFulfillItemsV3
from ..operations.fulfillment import RevokeItems
from ..operations.fulfillment import RevokeItemsV3
from ..models import FulfillmentItemItemTypeEnum
from ..models import (
    FulfillmentRequestEntitlementOriginEnum,
    FulfillmentRequestOriginEnum,
    FulfillmentRequestSourceEnum,
)
from ..models import FulfillmentV2ResultStateEnum
from ..models import RevokeFulfillmentV2ResultStateEnum
from ..models import (
    RewardsRequestEntitlementOriginEnum,
    RewardsRequestOriginEnum,
    RewardsRequestSourceEnum,
)


@same_doc_as(BulkFulfillItemsV3)
def bulk_fulfill_items_v3(
    body: List[FulfillmentV3Request],
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Fulfill multiple transactions (bulkFulfillItemsV3)

    [Not supported yet in AGS Shared Cloud] Fulfill multiple transactions.
    Other detail info:

      * Request body : list of fulfillment v3 requests. storeId, region, language, and entitlementCollectionId can be ignored.
      *  Returns : list of fulfillment v2 result

    Properties:
        url: /platform/v3/admin/namespaces/{namespace}/users/{userId}/fulfillments/bulk

        method: PUT

        tags: ["Fulfillment"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED List[FulfillmentV3Request] in body

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        200: OK - List[FulfillmentV2Result] (OK)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = BulkFulfillItemsV3.create(
        body=body,
        user_id=user_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(BulkFulfillItemsV3)
async def bulk_fulfill_items_v3_async(
    body: List[FulfillmentV3Request],
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Fulfill multiple transactions (bulkFulfillItemsV3)

    [Not supported yet in AGS Shared Cloud] Fulfill multiple transactions.
    Other detail info:

      * Request body : list of fulfillment v3 requests. storeId, region, language, and entitlementCollectionId can be ignored.
      *  Returns : list of fulfillment v2 result

    Properties:
        url: /platform/v3/admin/namespaces/{namespace}/users/{userId}/fulfillments/bulk

        method: PUT

        tags: ["Fulfillment"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED List[FulfillmentV3Request] in body

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        200: OK - List[FulfillmentV2Result] (OK)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = BulkFulfillItemsV3.create(
        body=body,
        user_id=user_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(FulfillItem)
def fulfill_item(
    body: FulfillmentRequest,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Fulfill item (fulfillItem)

    Fulfill item.
    Other detail info:

      * Returns : fulfillment result

    Properties:
        url: /platform/admin/namespaces/{namespace}/users/{userId}/fulfillment

        method: POST

        tags: ["Fulfillment"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED FulfillmentRequest in body

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        200: OK - FulfillmentResult (successful operation)

        400: Bad Request - ErrorEntity (35123: Wallet [{walletId}] is inactive | 38121: Duplicate permanent item exists | 38122: Subscription endDate required)

        404: Not Found - ErrorEntity (30341: Item [{itemId}] does not exist in namespace [{namespace}])

        409: Conflict - ErrorEntity (20006: optimistic lock)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = FulfillItem.create(
        body=body,
        user_id=user_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(FulfillItem)
async def fulfill_item_async(
    body: FulfillmentRequest,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Fulfill item (fulfillItem)

    Fulfill item.
    Other detail info:

      * Returns : fulfillment result

    Properties:
        url: /platform/admin/namespaces/{namespace}/users/{userId}/fulfillment

        method: POST

        tags: ["Fulfillment"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED FulfillmentRequest in body

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        200: OK - FulfillmentResult (successful operation)

        400: Bad Request - ErrorEntity (35123: Wallet [{walletId}] is inactive | 38121: Duplicate permanent item exists | 38122: Subscription endDate required)

        404: Not Found - ErrorEntity (30341: Item [{itemId}] does not exist in namespace [{namespace}])

        409: Conflict - ErrorEntity (20006: optimistic lock)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = FulfillItem.create(
        body=body,
        user_id=user_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@deprecated
@same_doc_as(FulfillItems)
def fulfill_items(
    body: FulfillmentV2Request,
    transaction_id: str,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Fulfill items by transactionId (fulfillItems)

    ### The endpoint is going to be deprecated

    Description: this endpoint is Not supported yet in AGS Shared Cloud and it's used to fulfill items by transaction id.

      *  Request body : storeId, region, language, and entitlementCollectionId can be ignored.
      *  Returns : fulfillment v2 result, storeId field can be ignored.



    ### Endpoint migration guide

      *  Substitute endpoint: /v3/admin/namespaces/{namespace}/users/{userId}/fulfillments/{transactionId} [PUT]

    Properties:
        url: /platform/v2/admin/namespaces/{namespace}/users/{userId}/fulfillments/{transactionId}

        method: PUT

        tags: ["Fulfillment"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED FulfillmentV2Request in body

        namespace: (namespace) REQUIRED str in path

        transaction_id: (transactionId) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        200: OK - FulfillmentV2Result (successful operation)

        400: Bad Request - ErrorEntity (35123: Wallet [{walletId}] is inactive | 38121: Duplicate permanent item exists | 38122: Subscription endDate required | 38128: Cannot retry fulfillment with different payload. Please check the items list. | 38129: Cannot combine same item [{itemId}] with different [{fieldName}] value | 38130: Cannot fulfill item with type [{itemType}] in item [{itemIdentity}])

        404: Not Found - ErrorEntity (30341: Item [{itemId}] does not exist in namespace [{namespace}] | 30343: Item of sku [{sku}] does not exist)

        409: Conflict - FulfillmentV2Result ()
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = FulfillItems.create(
        body=body,
        transaction_id=transaction_id,
        user_id=user_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(FulfillItems)
async def fulfill_items_async(
    body: FulfillmentV2Request,
    transaction_id: str,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Fulfill items by transactionId (fulfillItems)

    ### The endpoint is going to be deprecated

    Description: this endpoint is Not supported yet in AGS Shared Cloud and it's used to fulfill items by transaction id.

      *  Request body : storeId, region, language, and entitlementCollectionId can be ignored.
      *  Returns : fulfillment v2 result, storeId field can be ignored.



    ### Endpoint migration guide

      *  Substitute endpoint: /v3/admin/namespaces/{namespace}/users/{userId}/fulfillments/{transactionId} [PUT]

    Properties:
        url: /platform/v2/admin/namespaces/{namespace}/users/{userId}/fulfillments/{transactionId}

        method: PUT

        tags: ["Fulfillment"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED FulfillmentV2Request in body

        namespace: (namespace) REQUIRED str in path

        transaction_id: (transactionId) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        200: OK - FulfillmentV2Result (successful operation)

        400: Bad Request - ErrorEntity (35123: Wallet [{walletId}] is inactive | 38121: Duplicate permanent item exists | 38122: Subscription endDate required | 38128: Cannot retry fulfillment with different payload. Please check the items list. | 38129: Cannot combine same item [{itemId}] with different [{fieldName}] value | 38130: Cannot fulfill item with type [{itemType}] in item [{itemIdentity}])

        404: Not Found - ErrorEntity (30341: Item [{itemId}] does not exist in namespace [{namespace}] | 30343: Item of sku [{sku}] does not exist)

        409: Conflict - FulfillmentV2Result ()
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = FulfillItems.create(
        body=body,
        transaction_id=transaction_id,
        user_id=user_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(FulfillItemsV3)
def fulfill_items_v3(
    body: FulfillmentV2Request,
    transaction_id: str,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Fulfill items by transactionId (fulfillItemsV3)

    [Not supported yet in AGS Shared Cloud] Fulfill items by transactionId.
    Other detail info:

      * Request body : storeId, region, language, and entitlementCollectionId can be ignored.
      *  Returns : fulfillment v2 result, storeId field can be ignored.

    Properties:
        url: /platform/v3/admin/namespaces/{namespace}/users/{userId}/fulfillments/{transactionId}

        method: PUT

        tags: ["Fulfillment"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED FulfillmentV2Request in body

        namespace: (namespace) REQUIRED str in path

        transaction_id: (transactionId) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        200: OK - FulfillmentV2Result (successful operation)

        400: Bad Request - ErrorEntity (35123: Wallet [{walletId}] is inactive | 38121: Duplicate permanent item exists | 38122: Subscription endDate required | 38128: Cannot retry fulfillment with different payload. Please check the items list. | 38129: Cannot combine same item [{itemId}] with different [{fieldName}] value | 38130: Cannot fulfill item with type [{itemType}] in item [{itemIdentity}])

        404: Not Found - ErrorEntity (30341: Item [{itemId}] does not exist in namespace [{namespace}] | 30343: Item of sku [{sku}] does not exist)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = FulfillItemsV3.create(
        body=body,
        transaction_id=transaction_id,
        user_id=user_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(FulfillItemsV3)
async def fulfill_items_v3_async(
    body: FulfillmentV2Request,
    transaction_id: str,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Fulfill items by transactionId (fulfillItemsV3)

    [Not supported yet in AGS Shared Cloud] Fulfill items by transactionId.
    Other detail info:

      * Request body : storeId, region, language, and entitlementCollectionId can be ignored.
      *  Returns : fulfillment v2 result, storeId field can be ignored.

    Properties:
        url: /platform/v3/admin/namespaces/{namespace}/users/{userId}/fulfillments/{transactionId}

        method: PUT

        tags: ["Fulfillment"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED FulfillmentV2Request in body

        namespace: (namespace) REQUIRED str in path

        transaction_id: (transactionId) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        200: OK - FulfillmentV2Result (successful operation)

        400: Bad Request - ErrorEntity (35123: Wallet [{walletId}] is inactive | 38121: Duplicate permanent item exists | 38122: Subscription endDate required | 38128: Cannot retry fulfillment with different payload. Please check the items list. | 38129: Cannot combine same item [{itemId}] with different [{fieldName}] value | 38130: Cannot fulfill item with type [{itemType}] in item [{itemIdentity}])

        404: Not Found - ErrorEntity (30341: Item [{itemId}] does not exist in namespace [{namespace}] | 30343: Item of sku [{sku}] does not exist)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = FulfillItemsV3.create(
        body=body,
        transaction_id=transaction_id,
        user_id=user_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(FulfillRewards)
def fulfill_rewards(
    body: RewardsRequest,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Fulfill rewards without content (fulfillRewards)

    [SERVICE COMMUNICATION ONLY] Fulfill rewards.
    Other detail info:

      * Returns : fulfillment result

    Properties:
        url: /platform/admin/namespaces/{namespace}/users/{userId}/fulfillment/rewards

        method: POST

        tags: ["Fulfillment"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED RewardsRequest in body

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        204: No Content - (fulfill successfully)

        400: Bad Request - ErrorEntity (35123: Wallet [{walletId}] is inactive | 38121: Duplicate permanent item exists)

        404: Not Found - ErrorEntity (30341: Item [{itemId}] does not exist in namespace [{namespace}])

        409: Conflict - ErrorEntity (20006: optimistic lock)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = FulfillRewards.create(
        body=body,
        user_id=user_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(FulfillRewards)
async def fulfill_rewards_async(
    body: RewardsRequest,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Fulfill rewards without content (fulfillRewards)

    [SERVICE COMMUNICATION ONLY] Fulfill rewards.
    Other detail info:

      * Returns : fulfillment result

    Properties:
        url: /platform/admin/namespaces/{namespace}/users/{userId}/fulfillment/rewards

        method: POST

        tags: ["Fulfillment"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED RewardsRequest in body

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        204: No Content - (fulfill successfully)

        400: Bad Request - ErrorEntity (35123: Wallet [{walletId}] is inactive | 38121: Duplicate permanent item exists)

        404: Not Found - ErrorEntity (30341: Item [{itemId}] does not exist in namespace [{namespace}])

        409: Conflict - ErrorEntity (20006: optimistic lock)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = FulfillRewards.create(
        body=body,
        user_id=user_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(FulfillRewardsV2)
def fulfill_rewards_v2(
    body: RewardsRequest,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Fulfill rewards (fulfillRewardsV2)

    [SERVICE COMMUNICATION ONLY] Fulfill rewards.
    Other detail info:

      * Returns : fulfillment result
      *  rewards Item unsupported Type : SUBSCRIPTION

    Properties:
        url: /platform/v2/admin/namespaces/{namespace}/users/{userId}/fulfillment/rewards

        method: POST

        tags: ["Fulfillment"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED RewardsRequest in body

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        200: OK - FulfillmentResult (successful operation)

        400: Bad Request - ErrorEntity (35123: Wallet [{walletId}] is inactive | 38121: Duplicate permanent item exists)

        404: Not Found - ErrorEntity (30341: Item [{itemId}] does not exist in namespace [{namespace}])

        409: Conflict - ErrorEntity (20006: optimistic lock)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = FulfillRewardsV2.create(
        body=body,
        user_id=user_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(FulfillRewardsV2)
async def fulfill_rewards_v2_async(
    body: RewardsRequest,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Fulfill rewards (fulfillRewardsV2)

    [SERVICE COMMUNICATION ONLY] Fulfill rewards.
    Other detail info:

      * Returns : fulfillment result
      *  rewards Item unsupported Type : SUBSCRIPTION

    Properties:
        url: /platform/v2/admin/namespaces/{namespace}/users/{userId}/fulfillment/rewards

        method: POST

        tags: ["Fulfillment"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED RewardsRequest in body

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        200: OK - FulfillmentResult (successful operation)

        400: Bad Request - ErrorEntity (35123: Wallet [{walletId}] is inactive | 38121: Duplicate permanent item exists)

        404: Not Found - ErrorEntity (30341: Item [{itemId}] does not exist in namespace [{namespace}])

        409: Conflict - ErrorEntity (20006: optimistic lock)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = FulfillRewardsV2.create(
        body=body,
        user_id=user_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(PreCheckFulfillItem)
def pre_check_fulfill_item(
    body: PreCheckFulfillmentRequest,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Pre check fulfillment items (preCheckFulfillItem)

    Retrieve and check fulfillment items based on the provided request.
    Other detail info:

      * Returns : list of fulfillment items

    Properties:
        url: /platform/admin/namespaces/{namespace}/users/{userId}/fulfillment/preCheck

        method: POST

        tags: ["Fulfillment"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED PreCheckFulfillmentRequest in body

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        200: OK - List[FulfillmentItem] (Successful retrieval)

        400: Bad Request - ErrorEntity (38121: Duplicate permanent item exists)

        404: Not Found - ErrorEntity (30341: Item [{itemId}] does not exist in namespace [{namespace}])
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = PreCheckFulfillItem.create(
        body=body,
        user_id=user_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(PreCheckFulfillItem)
async def pre_check_fulfill_item_async(
    body: PreCheckFulfillmentRequest,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Pre check fulfillment items (preCheckFulfillItem)

    Retrieve and check fulfillment items based on the provided request.
    Other detail info:

      * Returns : list of fulfillment items

    Properties:
        url: /platform/admin/namespaces/{namespace}/users/{userId}/fulfillment/preCheck

        method: POST

        tags: ["Fulfillment"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED PreCheckFulfillmentRequest in body

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        200: OK - List[FulfillmentItem] (Successful retrieval)

        400: Bad Request - ErrorEntity (38121: Duplicate permanent item exists)

        404: Not Found - ErrorEntity (30341: Item [{itemId}] does not exist in namespace [{namespace}])
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = PreCheckFulfillItem.create(
        body=body,
        user_id=user_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(PublicRedeemCode)
def public_redeem_code(
    body: FulfillCodeRequest,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Redeem campaign code (publicRedeemCode)

    Redeem campaign code, this api have rate limit, default: only allow request once per user in 2 seconds
    Other detail info:

      * Returns : fulfillment result

    Properties:
        url: /platform/public/namespaces/{namespace}/users/{userId}/fulfillment/code

        method: POST

        tags: ["Fulfillment"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED FulfillCodeRequest in body

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        200: OK - FulfillmentResult (successful operation)

        400: Bad Request - ErrorEntity (35123: Wallet [{walletId}] is inactive | 38121: Duplicate permanent item exists)

        404: Not Found - ErrorEntity (30341: Item [{itemId}] does not exist in namespace [{namespace}] | 37142: Code [{code}] does not exist in namespace [{namespace}])

        409: Conflict - ErrorEntity (37172: Campaign [{campaignId}] is inactive in namespace [{namespace}] | 37173: Code [{code}] is inactive in namespace [{namespace}] | 37174: Exceeded max redeem count per code [{maxCount}] | 37175: Exceeded max redeem count per code per user [{maxCount}] | 37177: Code redemption not started | 37178: Code redemption already ended | 20006: optimistic lock | 31177: Permanent item already owned)

        429: Too Many Requests - ErrorEntity (20007: too many requests)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = PublicRedeemCode.create(
        body=body,
        user_id=user_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(PublicRedeemCode)
async def public_redeem_code_async(
    body: FulfillCodeRequest,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Redeem campaign code (publicRedeemCode)

    Redeem campaign code, this api have rate limit, default: only allow request once per user in 2 seconds
    Other detail info:

      * Returns : fulfillment result

    Properties:
        url: /platform/public/namespaces/{namespace}/users/{userId}/fulfillment/code

        method: POST

        tags: ["Fulfillment"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED FulfillCodeRequest in body

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        200: OK - FulfillmentResult (successful operation)

        400: Bad Request - ErrorEntity (35123: Wallet [{walletId}] is inactive | 38121: Duplicate permanent item exists)

        404: Not Found - ErrorEntity (30341: Item [{itemId}] does not exist in namespace [{namespace}] | 37142: Code [{code}] does not exist in namespace [{namespace}])

        409: Conflict - ErrorEntity (37172: Campaign [{campaignId}] is inactive in namespace [{namespace}] | 37173: Code [{code}] is inactive in namespace [{namespace}] | 37174: Exceeded max redeem count per code [{maxCount}] | 37175: Exceeded max redeem count per code per user [{maxCount}] | 37177: Code redemption not started | 37178: Code redemption already ended | 20006: optimistic lock | 31177: Permanent item already owned)

        429: Too Many Requests - ErrorEntity (20007: too many requests)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = PublicRedeemCode.create(
        body=body,
        user_id=user_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(QueryFulfillmentHistories)
def query_fulfillment_histories(
    user_id: str,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    status: Optional[Union[str, QueryFulfillmentHistoriesStatusEnum]] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Query fulfillment histories (queryFulfillmentHistories)

    Query fulfillment histories in a namespace.
    Other detail info:

      * Returns : query fulfillment history

    Properties:
        url: /platform/admin/namespaces/{namespace}/fulfillment/history

        method: GET

        tags: ["Fulfillment"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

        status: (status) OPTIONAL Union[str, StatusEnum] in query

        user_id: (userId) REQUIRED str in query

    Responses:
        200: OK - FulfillmentHistoryPagingSlicedResult (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = QueryFulfillmentHistories.create(
        user_id=user_id,
        limit=limit,
        offset=offset,
        status=status,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(QueryFulfillmentHistories)
async def query_fulfillment_histories_async(
    user_id: str,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    status: Optional[Union[str, QueryFulfillmentHistoriesStatusEnum]] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Query fulfillment histories (queryFulfillmentHistories)

    Query fulfillment histories in a namespace.
    Other detail info:

      * Returns : query fulfillment history

    Properties:
        url: /platform/admin/namespaces/{namespace}/fulfillment/history

        method: GET

        tags: ["Fulfillment"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

        status: (status) OPTIONAL Union[str, StatusEnum] in query

        user_id: (userId) REQUIRED str in query

    Responses:
        200: OK - FulfillmentHistoryPagingSlicedResult (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = QueryFulfillmentHistories.create(
        user_id=user_id,
        limit=limit,
        offset=offset,
        status=status,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(QueryFulfillments)
def query_fulfillments(
    end_time: Optional[str] = None,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    start_time: Optional[str] = None,
    state: Optional[Union[str, QueryFulfillmentsStateEnum]] = None,
    transaction_id: Optional[str] = None,
    user_id: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Query fulfillments (queryFulfillments)

    [Not supported yet in AGS Shared Cloud] Query fulfillments in a namespace.
    Other detail info:

      * Returns : list of fulfillment info:
        * storeId in items can be ignored
        * error in successList will always be null

    Properties:
        url: /platform/v2/admin/namespaces/{namespace}/fulfillments

        method: GET

        tags: ["Fulfillment"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        end_time: (endTime) OPTIONAL str in query

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

        start_time: (startTime) OPTIONAL str in query

        state: (state) OPTIONAL Union[str, StateEnum] in query

        transaction_id: (transactionId) OPTIONAL str in query

        user_id: (userId) OPTIONAL str in query

    Responses:
        200: OK - FulfillmentPagingSlicedResult (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = QueryFulfillments.create(
        end_time=end_time,
        limit=limit,
        offset=offset,
        start_time=start_time,
        state=state,
        transaction_id=transaction_id,
        user_id=user_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(QueryFulfillments)
async def query_fulfillments_async(
    end_time: Optional[str] = None,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    start_time: Optional[str] = None,
    state: Optional[Union[str, QueryFulfillmentsStateEnum]] = None,
    transaction_id: Optional[str] = None,
    user_id: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Query fulfillments (queryFulfillments)

    [Not supported yet in AGS Shared Cloud] Query fulfillments in a namespace.
    Other detail info:

      * Returns : list of fulfillment info:
        * storeId in items can be ignored
        * error in successList will always be null

    Properties:
        url: /platform/v2/admin/namespaces/{namespace}/fulfillments

        method: GET

        tags: ["Fulfillment"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        end_time: (endTime) OPTIONAL str in query

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

        start_time: (startTime) OPTIONAL str in query

        state: (state) OPTIONAL Union[str, StateEnum] in query

        transaction_id: (transactionId) OPTIONAL str in query

        user_id: (userId) OPTIONAL str in query

    Responses:
        200: OK - FulfillmentPagingSlicedResult (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = QueryFulfillments.create(
        end_time=end_time,
        limit=limit,
        offset=offset,
        start_time=start_time,
        state=state,
        transaction_id=transaction_id,
        user_id=user_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(RedeemCode)
def redeem_code(
    body: FulfillCodeRequest,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Redeem campaign code (redeemCode)

    Redeem campaign code.
    Other detail info:

      * Returns : fulfillment result

    Properties:
        url: /platform/admin/namespaces/{namespace}/users/{userId}/fulfillment/code

        method: POST

        tags: ["Fulfillment"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED FulfillCodeRequest in body

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        200: OK - FulfillmentResult (successful operation)

        400: Bad Request - ErrorEntity (35123: Wallet [{walletId}] is inactive | 38121: Duplicate permanent item exists)

        404: Not Found - ErrorEntity (30341: Item [{itemId}] does not exist in namespace [{namespace}] | 37142: Code [{code}] does not exist in namespace [{namespace}])

        409: Conflict - ErrorEntity (37172: Campaign [{campaignId}] is inactive in namespace [{namespace}] | 37173: Code [{code}] is inactive in namespace [{namespace}] | 37174: Exceeded max redeem count per code [{maxCount}] | 37175: Exceeded max redeem count per code per user [{maxCount}] | 37177: Code redemption not started | 37178: Code redemption already ended | 20006: optimistic lock | 31177: Permanent item already owned)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = RedeemCode.create(
        body=body,
        user_id=user_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(RedeemCode)
async def redeem_code_async(
    body: FulfillCodeRequest,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Redeem campaign code (redeemCode)

    Redeem campaign code.
    Other detail info:

      * Returns : fulfillment result

    Properties:
        url: /platform/admin/namespaces/{namespace}/users/{userId}/fulfillment/code

        method: POST

        tags: ["Fulfillment"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED FulfillCodeRequest in body

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        200: OK - FulfillmentResult (successful operation)

        400: Bad Request - ErrorEntity (35123: Wallet [{walletId}] is inactive | 38121: Duplicate permanent item exists)

        404: Not Found - ErrorEntity (30341: Item [{itemId}] does not exist in namespace [{namespace}] | 37142: Code [{code}] does not exist in namespace [{namespace}])

        409: Conflict - ErrorEntity (37172: Campaign [{campaignId}] is inactive in namespace [{namespace}] | 37173: Code [{code}] is inactive in namespace [{namespace}] | 37174: Exceeded max redeem count per code [{maxCount}] | 37175: Exceeded max redeem count per code per user [{maxCount}] | 37177: Code redemption not started | 37178: Code redemption already ended | 20006: optimistic lock | 31177: Permanent item already owned)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = RedeemCode.create(
        body=body,
        user_id=user_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@deprecated
@same_doc_as(RetryFulfillItems)
def retry_fulfill_items(
    transaction_id: str,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Retry fulfill items by transactionId (retryFulfillItems)

    ### The endpoint is going to be deprecated

    [Not supported yet in AGS Shared Cloud] Retry fulfill items by transactionId without sending the original payload.
    Other detail info:

      * Returns : fulfillment v2 result, storeId field can be ignored.



    ### Endpoint migration guide

      *  Substitute endpoint: /v3/admin/namespaces/{namespace}/users/{userId}/fulfillments/{transactionId}/retry [PUT]

    Properties:
        url: /platform/v2/admin/namespaces/{namespace}/users/{userId}/fulfillments/{transactionId}/retry

        method: PUT

        tags: ["Fulfillment"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        transaction_id: (transactionId) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        200: OK - FulfillmentV2Result (successful operation)

        404: Not Found - ErrorEntity (38145: Fulfillment with transactionId [{transactionId}] does not exist)

        409: Conflict - FulfillmentV2Result ()
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = RetryFulfillItems.create(
        transaction_id=transaction_id,
        user_id=user_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(RetryFulfillItems)
async def retry_fulfill_items_async(
    transaction_id: str,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Retry fulfill items by transactionId (retryFulfillItems)

    ### The endpoint is going to be deprecated

    [Not supported yet in AGS Shared Cloud] Retry fulfill items by transactionId without sending the original payload.
    Other detail info:

      * Returns : fulfillment v2 result, storeId field can be ignored.



    ### Endpoint migration guide

      *  Substitute endpoint: /v3/admin/namespaces/{namespace}/users/{userId}/fulfillments/{transactionId}/retry [PUT]

    Properties:
        url: /platform/v2/admin/namespaces/{namespace}/users/{userId}/fulfillments/{transactionId}/retry

        method: PUT

        tags: ["Fulfillment"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        transaction_id: (transactionId) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        200: OK - FulfillmentV2Result (successful operation)

        404: Not Found - ErrorEntity (38145: Fulfillment with transactionId [{transactionId}] does not exist)

        409: Conflict - FulfillmentV2Result ()
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = RetryFulfillItems.create(
        transaction_id=transaction_id,
        user_id=user_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(RetryFulfillItemsV3)
def retry_fulfill_items_v3(
    transaction_id: str,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Retry fulfill items by transactionId (retryFulfillItemsV3)

    [Not supported yet in AGS Shared Cloud] Retry fulfill items by transactionId without sending the original payload.
    Other detail info:

      * Returns : fulfillment v2 result, storeId field can be ignored.

    Properties:
        url: /platform/v3/admin/namespaces/{namespace}/users/{userId}/fulfillments/{transactionId}/retry

        method: PUT

        tags: ["Fulfillment"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        transaction_id: (transactionId) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        200: OK - FulfillmentV2Result (successful operation)

        404: Not Found - ErrorEntity (38145: Fulfillment with transactionId [{transactionId}] does not exist)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = RetryFulfillItemsV3.create(
        transaction_id=transaction_id,
        user_id=user_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(RetryFulfillItemsV3)
async def retry_fulfill_items_v3_async(
    transaction_id: str,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Retry fulfill items by transactionId (retryFulfillItemsV3)

    [Not supported yet in AGS Shared Cloud] Retry fulfill items by transactionId without sending the original payload.
    Other detail info:

      * Returns : fulfillment v2 result, storeId field can be ignored.

    Properties:
        url: /platform/v3/admin/namespaces/{namespace}/users/{userId}/fulfillments/{transactionId}/retry

        method: PUT

        tags: ["Fulfillment"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        transaction_id: (transactionId) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        200: OK - FulfillmentV2Result (successful operation)

        404: Not Found - ErrorEntity (38145: Fulfillment with transactionId [{transactionId}] does not exist)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = RetryFulfillItemsV3.create(
        transaction_id=transaction_id,
        user_id=user_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@deprecated
@same_doc_as(RevokeItems)
def revoke_items(
    transaction_id: str,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Revoke items by transactionId (revokeItems)

    ### The endpoint is going to be deprecated

    [Not supported yet in AGS Shared Cloud] Revoke items by transactionId.
    Other detail info:

      * Returns : revoke fulfillment v2 result, storeId field can be ignored.



    ### Endpoint migration guide

      *  Substitute endpoint: /v3/admin/namespaces/{namespace}/users/{userId}/fulfillments/{transactionId}/revoke [PUT]

    Properties:
        url: /platform/v2/admin/namespaces/{namespace}/users/{userId}/fulfillments/{transactionId}/revoke

        method: PUT

        tags: ["Fulfillment"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        transaction_id: (transactionId) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        200: OK - RevokeFulfillmentV2Result (successful operation)

        404: Not Found - ErrorEntity (38145: Fulfillment with transactionId [{transactionId}] does not exist)

        409: Conflict - RevokeFulfillmentV2Result ()
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = RevokeItems.create(
        transaction_id=transaction_id,
        user_id=user_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(RevokeItems)
async def revoke_items_async(
    transaction_id: str,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Revoke items by transactionId (revokeItems)

    ### The endpoint is going to be deprecated

    [Not supported yet in AGS Shared Cloud] Revoke items by transactionId.
    Other detail info:

      * Returns : revoke fulfillment v2 result, storeId field can be ignored.



    ### Endpoint migration guide

      *  Substitute endpoint: /v3/admin/namespaces/{namespace}/users/{userId}/fulfillments/{transactionId}/revoke [PUT]

    Properties:
        url: /platform/v2/admin/namespaces/{namespace}/users/{userId}/fulfillments/{transactionId}/revoke

        method: PUT

        tags: ["Fulfillment"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        transaction_id: (transactionId) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        200: OK - RevokeFulfillmentV2Result (successful operation)

        404: Not Found - ErrorEntity (38145: Fulfillment with transactionId [{transactionId}] does not exist)

        409: Conflict - RevokeFulfillmentV2Result ()
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = RevokeItems.create(
        transaction_id=transaction_id,
        user_id=user_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(RevokeItemsV3)
def revoke_items_v3(
    transaction_id: str,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Revoke items by transactionId (revokeItemsV3)

    [Not supported yet in AGS Shared Cloud] Revoke items by transactionId.
    Other detail info:

      * Returns : revoke fulfillment v2 result, storeId field can be ignored.

    Properties:
        url: /platform/v3/admin/namespaces/{namespace}/users/{userId}/fulfillments/{transactionId}/revoke

        method: PUT

        tags: ["Fulfillment"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        transaction_id: (transactionId) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        200: OK - RevokeFulfillmentV2Result (successful operation)

        404: Not Found - ErrorEntity (38145: Fulfillment with transactionId [{transactionId}] does not exist)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = RevokeItemsV3.create(
        transaction_id=transaction_id,
        user_id=user_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(RevokeItemsV3)
async def revoke_items_v3_async(
    transaction_id: str,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Revoke items by transactionId (revokeItemsV3)

    [Not supported yet in AGS Shared Cloud] Revoke items by transactionId.
    Other detail info:

      * Returns : revoke fulfillment v2 result, storeId field can be ignored.

    Properties:
        url: /platform/v3/admin/namespaces/{namespace}/users/{userId}/fulfillments/{transactionId}/revoke

        method: PUT

        tags: ["Fulfillment"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        transaction_id: (transactionId) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        200: OK - RevokeFulfillmentV2Result (successful operation)

        404: Not Found - ErrorEntity (38145: Fulfillment with transactionId [{transactionId}] does not exist)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = RevokeItemsV3.create(
        transaction_id=transaction_id,
        user_id=user_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# Custom Service Manager

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class ApimodelAppItemV5(Model):
    """Apimodel app item V5 (apimodel.AppItemV5)

    Properties:
        app_id: (appId) REQUIRED str

        app_name: (appName) REQUIRED str

        app_status: (appStatus) REQUIRED str

        created_at: (createdAt) REQUIRED str

        game_name: (gameName) REQUIRED str

        scenario: (scenario) REQUIRED str

        updated_at: (updatedAt) REQUIRED str

        base_path: (basePath) OPTIONAL str

        description: (description) OPTIONAL str

        vm_sharing_configuration: (vmSharingConfiguration) OPTIONAL str
    """

    # region fields

    app_id: str  # REQUIRED
    app_name: str  # REQUIRED
    app_status: str  # REQUIRED
    created_at: str  # REQUIRED
    game_name: str  # REQUIRED
    scenario: str  # REQUIRED
    updated_at: str  # REQUIRED
    base_path: str  # OPTIONAL
    description: str  # OPTIONAL
    vm_sharing_configuration: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_app_id(self, value: str) -> ApimodelAppItemV5:
        self.app_id = value
        return self

    def with_app_name(self, value: str) -> ApimodelAppItemV5:
        self.app_name = value
        return self

    def with_app_status(self, value: str) -> ApimodelAppItemV5:
        self.app_status = value
        return self

    def with_created_at(self, value: str) -> ApimodelAppItemV5:
        self.created_at = value
        return self

    def with_game_name(self, value: str) -> ApimodelAppItemV5:
        self.game_name = value
        return self

    def with_scenario(self, value: str) -> ApimodelAppItemV5:
        self.scenario = value
        return self

    def with_updated_at(self, value: str) -> ApimodelAppItemV5:
        self.updated_at = value
        return self

    def with_base_path(self, value: str) -> ApimodelAppItemV5:
        self.base_path = value
        return self

    def with_description(self, value: str) -> ApimodelAppItemV5:
        self.description = value
        return self

    def with_vm_sharing_configuration(self, value: str) -> ApimodelAppItemV5:
        self.vm_sharing_configuration = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "app_id"):
            result["appId"] = str(self.app_id)
        elif include_empty:
            result["appId"] = ""
        if hasattr(self, "app_name"):
            result["appName"] = str(self.app_name)
        elif include_empty:
            result["appName"] = ""
        if hasattr(self, "app_status"):
            result["appStatus"] = str(self.app_status)
        elif include_empty:
            result["appStatus"] = ""
        if hasattr(self, "created_at"):
            result["createdAt"] = str(self.created_at)
        elif include_empty:
            result["createdAt"] = ""
        if hasattr(self, "game_name"):
            result["gameName"] = str(self.game_name)
        elif include_empty:
            result["gameName"] = ""
        if hasattr(self, "scenario"):
            result["scenario"] = str(self.scenario)
        elif include_empty:
            result["scenario"] = ""
        if hasattr(self, "updated_at"):
            result["updatedAt"] = str(self.updated_at)
        elif include_empty:
            result["updatedAt"] = ""
        if hasattr(self, "base_path"):
            result["basePath"] = str(self.base_path)
        elif include_empty:
            result["basePath"] = ""
        if hasattr(self, "description"):
            result["description"] = str(self.description)
        elif include_empty:
            result["description"] = ""
        if hasattr(self, "vm_sharing_configuration"):
            result["vmSharingConfiguration"] = str(self.vm_sharing_configuration)
        elif include_empty:
            result["vmSharingConfiguration"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        app_id: str,
        app_name: str,
        app_status: str,
        created_at: str,
        game_name: str,
        scenario: str,
        updated_at: str,
        base_path: Optional[str] = None,
        description: Optional[str] = None,
        vm_sharing_configuration: Optional[str] = None,
        **kwargs,
    ) -> ApimodelAppItemV5:
        instance = cls()
        instance.app_id = app_id
        instance.app_name = app_name
        instance.app_status = app_status
        instance.created_at = created_at
        instance.game_name = game_name
        instance.scenario = scenario
        instance.updated_at = updated_at
        if base_path is not None:
            instance.base_path = base_path
        if description is not None:
            instance.description = description
        if vm_sharing_configuration is not None:
            instance.vm_sharing_configuration = vm_sharing_configuration
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ApimodelAppItemV5:
        instance = cls()
        if not dict_:
            return instance
        if "appId" in dict_ and dict_["appId"] is not None:
            instance.app_id = str(dict_["appId"])
        elif include_empty:
            instance.app_id = ""
        if "appName" in dict_ and dict_["appName"] is not None:
            instance.app_name = str(dict_["appName"])
        elif include_empty:
            instance.app_name = ""
        if "appStatus" in dict_ and dict_["appStatus"] is not None:
            instance.app_status = str(dict_["appStatus"])
        elif include_empty:
            instance.app_status = ""
        if "createdAt" in dict_ and dict_["createdAt"] is not None:
            instance.created_at = str(dict_["createdAt"])
        elif include_empty:
            instance.created_at = ""
        if "gameName" in dict_ and dict_["gameName"] is not None:
            instance.game_name = str(dict_["gameName"])
        elif include_empty:
            instance.game_name = ""
        if "scenario" in dict_ and dict_["scenario"] is not None:
            instance.scenario = str(dict_["scenario"])
        elif include_empty:
            instance.scenario = ""
        if "updatedAt" in dict_ and dict_["updatedAt"] is not None:
            instance.updated_at = str(dict_["updatedAt"])
        elif include_empty:
            instance.updated_at = ""
        if "basePath" in dict_ and dict_["basePath"] is not None:
            instance.base_path = str(dict_["basePath"])
        elif include_empty:
            instance.base_path = ""
        if "description" in dict_ and dict_["description"] is not None:
            instance.description = str(dict_["description"])
        elif include_empty:
            instance.description = ""
        if (
            "vmSharingConfiguration" in dict_
            and dict_["vmSharingConfiguration"] is not None
        ):
            instance.vm_sharing_configuration = str(dict_["vmSharingConfiguration"])
        elif include_empty:
            instance.vm_sharing_configuration = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ApimodelAppItemV5]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ApimodelAppItemV5]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ApimodelAppItemV5, List[ApimodelAppItemV5], Dict[Any, ApimodelAppItemV5]
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "appId": "app_id",
            "appName": "app_name",
            "appStatus": "app_status",
            "createdAt": "created_at",
            "gameName": "game_name",
            "scenario": "scenario",
            "updatedAt": "updated_at",
            "basePath": "base_path",
            "description": "description",
            "vmSharingConfiguration": "vm_sharing_configuration",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "appId": True,
            "appName": True,
            "appStatus": True,
            "createdAt": True,
            "gameName": True,
            "scenario": True,
            "updatedAt": True,
            "basePath": False,
            "description": False,
            "vmSharingConfiguration": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Gdpr Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model
from ....core import StrEnum


class StatusEnum(StrEnum):
    COMPLETED = "Completed"
    FAILED = "Failed"


class DtoFinishedDataRequest(Model):
    """Dto finished data request (dto.FinishedDataRequest)

    Properties:
        finished_date: (finishedDate) REQUIRED str

        request_date: (requestDate) REQUIRED str

        request_id: (requestId) REQUIRED str

        status: (status) REQUIRED Union[str, StatusEnum]

        user_id: (userId) REQUIRED str

        data_expiration_date: (dataExpirationDate) OPTIONAL str

        failed_message: (failedMessage) OPTIONAL str
    """

    # region fields

    finished_date: str  # REQUIRED
    request_date: str  # REQUIRED
    request_id: str  # REQUIRED
    status: Union[str, StatusEnum]  # REQUIRED
    user_id: str  # REQUIRED
    data_expiration_date: str  # OPTIONAL
    failed_message: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_finished_date(self, value: str) -> DtoFinishedDataRequest:
        self.finished_date = value
        return self

    def with_request_date(self, value: str) -> DtoFinishedDataRequest:
        self.request_date = value
        return self

    def with_request_id(self, value: str) -> DtoFinishedDataRequest:
        self.request_id = value
        return self

    def with_status(self, value: Union[str, StatusEnum]) -> DtoFinishedDataRequest:
        self.status = value
        return self

    def with_user_id(self, value: str) -> DtoFinishedDataRequest:
        self.user_id = value
        return self

    def with_data_expiration_date(self, value: str) -> DtoFinishedDataRequest:
        self.data_expiration_date = value
        return self

    def with_failed_message(self, value: str) -> DtoFinishedDataRequest:
        self.failed_message = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "finished_date"):
            result["finishedDate"] = str(self.finished_date)
        elif include_empty:
            result["finishedDate"] = ""
        if hasattr(self, "request_date"):
            result["requestDate"] = str(self.request_date)
        elif include_empty:
            result["requestDate"] = ""
        if hasattr(self, "request_id"):
            result["requestId"] = str(self.request_id)
        elif include_empty:
            result["requestId"] = ""
        if hasattr(self, "status"):
            result["status"] = str(self.status)
        elif include_empty:
            result["status"] = Union[str, StatusEnum]()
        if hasattr(self, "user_id"):
            result["userId"] = str(self.user_id)
        elif include_empty:
            result["userId"] = ""
        if hasattr(self, "data_expiration_date"):
            result["dataExpirationDate"] = str(self.data_expiration_date)
        elif include_empty:
            result["dataExpirationDate"] = ""
        if hasattr(self, "failed_message"):
            result["failedMessage"] = str(self.failed_message)
        elif include_empty:
            result["failedMessage"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        finished_date: str,
        request_date: str,
        request_id: str,
        status: Union[str, StatusEnum],
        user_id: str,
        data_expiration_date: Optional[str] = None,
        failed_message: Optional[str] = None,
        **kwargs,
    ) -> DtoFinishedDataRequest:
        instance = cls()
        instance.finished_date = finished_date
        instance.request_date = request_date
        instance.request_id = request_id
        instance.status = status
        instance.user_id = user_id
        if data_expiration_date is not None:
            instance.data_expiration_date = data_expiration_date
        if failed_message is not None:
            instance.failed_message = failed_message
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> DtoFinishedDataRequest:
        instance = cls()
        if not dict_:
            return instance
        if "finishedDate" in dict_ and dict_["finishedDate"] is not None:
            instance.finished_date = str(dict_["finishedDate"])
        elif include_empty:
            instance.finished_date = ""
        if "requestDate" in dict_ and dict_["requestDate"] is not None:
            instance.request_date = str(dict_["requestDate"])
        elif include_empty:
            instance.request_date = ""
        if "requestId" in dict_ and dict_["requestId"] is not None:
            instance.request_id = str(dict_["requestId"])
        elif include_empty:
            instance.request_id = ""
        if "status" in dict_ and dict_["status"] is not None:
            instance.status = str(dict_["status"])
        elif include_empty:
            instance.status = Union[str, StatusEnum]()
        if "userId" in dict_ and dict_["userId"] is not None:
            instance.user_id = str(dict_["userId"])
        elif include_empty:
            instance.user_id = ""
        if "dataExpirationDate" in dict_ and dict_["dataExpirationDate"] is not None:
            instance.data_expiration_date = str(dict_["dataExpirationDate"])
        elif include_empty:
            instance.data_expiration_date = ""
        if "failedMessage" in dict_ and dict_["failedMessage"] is not None:
            instance.failed_message = str(dict_["failedMessage"])
        elif include_empty:
            instance.failed_message = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, DtoFinishedDataRequest]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[DtoFinishedDataRequest]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        DtoFinishedDataRequest,
        List[DtoFinishedDataRequest],
        Dict[Any, DtoFinishedDataRequest],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "finishedDate": "finished_date",
            "requestDate": "request_date",
            "requestId": "request_id",
            "status": "status",
            "userId": "user_id",
            "dataExpirationDate": "data_expiration_date",
            "failedMessage": "failed_message",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "finishedDate": True,
            "requestDate": True,
            "requestId": True,
            "status": True,
            "userId": True,
            "dataExpirationDate": False,
            "failedMessage": False,
        }

    @staticmethod
    def get_enum_map() -> Dict[str, List[Any]]:
        return {
            "status": ["Completed", "Failed"],
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Inventory Service Early Access

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.apimodels_inventory_config import ApimodelsInventoryConfig


class ApimodelsPurchaseValidationItemReq(Model):
    """Apimodels purchase validation item req (apimodels.PurchaseValidationItemReq)

    Properties:
        bundled_qty: (bundledQty) REQUIRED int

        entitlement_type: (entitlementType) REQUIRED str

        inventory_config: (inventoryConfig) REQUIRED ApimodelsInventoryConfig

        item_id: (itemId) REQUIRED str

        item_type: (itemType) REQUIRED str

        sku: (sku) REQUIRED str

        stackable: (stackable) REQUIRED bool

        use_count: (useCount) OPTIONAL int
    """

    # region fields

    bundled_qty: int  # REQUIRED
    entitlement_type: str  # REQUIRED
    inventory_config: ApimodelsInventoryConfig  # REQUIRED
    item_id: str  # REQUIRED
    item_type: str  # REQUIRED
    sku: str  # REQUIRED
    stackable: bool  # REQUIRED
    use_count: int  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_bundled_qty(self, value: int) -> ApimodelsPurchaseValidationItemReq:
        self.bundled_qty = value
        return self

    def with_entitlement_type(self, value: str) -> ApimodelsPurchaseValidationItemReq:
        self.entitlement_type = value
        return self

    def with_inventory_config(
        self, value: ApimodelsInventoryConfig
    ) -> ApimodelsPurchaseValidationItemReq:
        self.inventory_config = value
        return self

    def with_item_id(self, value: str) -> ApimodelsPurchaseValidationItemReq:
        self.item_id = value
        return self

    def with_item_type(self, value: str) -> ApimodelsPurchaseValidationItemReq:
        self.item_type = value
        return self

    def with_sku(self, value: str) -> ApimodelsPurchaseValidationItemReq:
        self.sku = value
        return self

    def with_stackable(self, value: bool) -> ApimodelsPurchaseValidationItemReq:
        self.stackable = value
        return self

    def with_use_count(self, value: int) -> ApimodelsPurchaseValidationItemReq:
        self.use_count = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "bundled_qty"):
            result["bundledQty"] = int(self.bundled_qty)
        elif include_empty:
            result["bundledQty"] = 0
        if hasattr(self, "entitlement_type"):
            result["entitlementType"] = str(self.entitlement_type)
        elif include_empty:
            result["entitlementType"] = ""
        if hasattr(self, "inventory_config"):
            result["inventoryConfig"] = self.inventory_config.to_dict(
                include_empty=include_empty
            )
        elif include_empty:
            result["inventoryConfig"] = ApimodelsInventoryConfig()
        if hasattr(self, "item_id"):
            result["itemId"] = str(self.item_id)
        elif include_empty:
            result["itemId"] = ""
        if hasattr(self, "item_type"):
            result["itemType"] = str(self.item_type)
        elif include_empty:
            result["itemType"] = ""
        if hasattr(self, "sku"):
            result["sku"] = str(self.sku)
        elif include_empty:
            result["sku"] = ""
        if hasattr(self, "stackable"):
            result["stackable"] = bool(self.stackable)
        elif include_empty:
            result["stackable"] = False
        if hasattr(self, "use_count"):
            result["useCount"] = int(self.use_count)
        elif include_empty:
            result["useCount"] = 0
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        bundled_qty: int,
        entitlement_type: str,
        inventory_config: ApimodelsInventoryConfig,
        item_id: str,
        item_type: str,
        sku: str,
        stackable: bool,
        use_count: Optional[int] = None,
        **kwargs,
    ) -> ApimodelsPurchaseValidationItemReq:
        instance = cls()
        instance.bundled_qty = bundled_qty
        instance.entitlement_type = entitlement_type
        instance.inventory_config = inventory_config
        instance.item_id = item_id
        instance.item_type = item_type
        instance.sku = sku
        instance.stackable = stackable
        if use_count is not None:
            instance.use_count = use_count
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ApimodelsPurchaseValidationItemReq:
        instance = cls()
        if not dict_:
            return instance
        if "bundledQty" in dict_ and dict_["bundledQty"] is not None:
            instance.bundled_qty = int(dict_["bundledQty"])
        elif include_empty:
            instance.bundled_qty = 0
        if "entitlementType" in dict_ and dict_["entitlementType"] is not None:
            instance.entitlement_type = str(dict_["entitlementType"])
        elif include_empty:
            instance.entitlement_type = ""
        if "inventoryConfig" in dict_ and dict_["inventoryConfig"] is not None:
            instance.inventory_config = ApimodelsInventoryConfig.create_from_dict(
                dict_["inventoryConfig"], include_empty=include_empty
            )
        elif include_empty:
            instance.inventory_config = ApimodelsInventoryConfig()
        if "itemId" in dict_ and dict_["itemId"] is not None:
            instance.item_id = str(dict_["itemId"])
        elif include_empty:
            instance.item_id = ""
        if "itemType" in dict_ and dict_["itemType"] is not None:
            instance.item_type = str(dict_["itemType"])
        elif include_empty:
            instance.item_type = ""
        if "sku" in dict_ and dict_["sku"] is not None:
            instance.sku = str(dict_["sku"])
        elif include_empty:
            instance.sku = ""
        if "stackable" in dict_ and dict_["stackable"] is not None:
            instance.stackable = bool(dict_["stackable"])
        elif include_empty:
            instance.stackable = False
        if "useCount" in dict_ and dict_["useCount"] is not None:
            instance.use_count = int(dict_["useCount"])
        elif include_empty:
            instance.use_count = 0
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ApimodelsPurchaseValidationItemReq]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ApimodelsPurchaseValidationItemReq]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ApimodelsPurchaseValidationItemReq,
        List[ApimodelsPurchaseValidationItemReq],
        Dict[Any, ApimodelsPurchaseValidationItemReq],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "bundledQty": "bundled_qty",
            "entitlementType": "entitlement_type",
            "inventoryConfig": "inventory_config",
            "itemId": "item_id",
            "itemType": "item_type",
            "sku": "sku",
            "stackable": "stackable",
            "useCount": "use_count",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "bundledQty": True,
            "entitlementType": True,
            "inventoryConfig": True,
            "itemId": True,
            "itemType": True,
            "sku": True,
            "stackable": True,
            "useCount": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

# AccelByte Gaming Services Iam Service

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from .....core import Operation
from .....core import HeaderStr
from .....core import HttpResponse

from ...models import OauthmodelTokenResponseV3


class RequestTokenByOneTimeLinkCodeResponseV3(Operation):
    """Generate Token by Headless Account's One-Time Link Code (RequestTokenByOneTimeLinkCodeResponseV3)

    Generates the user's token by one-time link code.
    It requires a code which can be generated from `/iam/v3/link/code/request [POST]` or `/iam/v3/public/users/me/link/redirection [GET]`.

    Supports creating a transient token by utilizing the **isTransient** param:
    **isTransient=true** will generate a transient token with a short Time Expiration and without a refresh token
    **isTransient=false** will consume the one-time code and generate the access token with a refresh token.

    Properties:
        url: /iam/v3/link/token/exchange

        method: POST

        tags: ["OAuth2.0 - Extension"]

        consumes: ["application/x-www-form-urlencoded"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        additional_data: (additionalData) OPTIONAL str in form_data

        is_transient: (isTransient) OPTIONAL bool in form_data

        client_id: (client_id) REQUIRED str in form_data

        one_time_link_code: (oneTimeLinkCode) REQUIRED str in form_data

    Responses:
        200: OK - OauthmodelTokenResponseV3 (Succeed to generate user token by one time link code.)
    """

    # region fields

    _url: str = "/iam/v3/link/token/exchange"
    _method: str = "POST"
    _consumes: List[str] = ["application/x-www-form-urlencoded"]
    _produces: List[str] = ["application/json"]
    _securities: List[List[str]] = [["BEARER_AUTH"]]
    _location_query: str = None

    additional_data: str  # OPTIONAL in [form_data]
    is_transient: bool  # OPTIONAL in [form_data]
    client_id: str  # REQUIRED in [form_data]
    one_time_link_code: str  # REQUIRED in [form_data]

    # endregion fields

    # region properties

    @property
    def url(self) -> str:
        return self._url

    @property
    def method(self) -> str:
        return self._method

    @property
    def consumes(self) -> List[str]:
        return self._consumes

    @property
    def produces(self) -> List[str]:
        return self._produces

    @property
    def securities(self) -> List[List[str]]:
        return self._securities

    @property
    def location_query(self) -> str:
        return self._location_query

    # endregion properties

    # region get methods

    # endregion get methods

    # region get_x_params methods

    def get_all_params(self) -> dict:
        return {
            "form_data": self.get_form_data_params(),
        }

    def get_form_data_params(self) -> dict:
        result = {}
        if hasattr(self, "additional_data"):
            result["additionalData"] = self.additional_data
        if hasattr(self, "is_transient"):
            result["isTransient"] = self.is_transient
        if hasattr(self, "client_id"):
            result["client_id"] = self.client_id
        if hasattr(self, "one_time_link_code"):
            result["oneTimeLinkCode"] = self.one_time_link_code
        return result

    # endregion get_x_params methods

    # region is/has methods

    # endregion is/has methods

    # region with_x methods

    def with_additional_data(
        self, value: str
    ) -> RequestTokenByOneTimeLinkCodeResponseV3:
        self.additional_data = value
        return self

    def with_is_transient(self, value: bool) -> RequestTokenByOneTimeLinkCodeResponseV3:
        self.is_transient = value
        return self

    def with_client_id(self, value: str) -> RequestTokenByOneTimeLinkCodeResponseV3:
        self.client_id = value
        return self

    def with_one_time_link_code(
        self, value: str
    ) -> RequestTokenByOneTimeLinkCodeResponseV3:
        self.one_time_link_code = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "additional_data") and self.additional_data:
            result["additionalData"] = str(self.additional_data)
        elif include_empty:
            result["additionalData"] = ""
        if hasattr(self, "is_transient") and self.is_transient:
            result["isTransient"] = bool(self.is_transient)
        elif include_empty:
            result["isTransient"] = False
        if hasattr(self, "client_id") and self.client_id:
            result["client_id"] = str(self.client_id)
        elif include_empty:
            result["client_id"] = ""
        if hasattr(self, "one_time_link_code") and self.one_time_link_code:
            result["oneTimeLinkCode"] = str(self.one_time_link_code)
        elif include_empty:
            result["oneTimeLinkCode"] = ""
        return result

    # endregion to methods

    # region response methods

    # noinspection PyMethodMayBeStatic
    def parse_response(
        self, code: int, content_type: str, content: Any
    ) -> Tuple[Union[None, OauthmodelTokenResponseV3], Union[None, HttpResponse]]:
        """Parse the given response.

        200: OK - OauthmodelTokenResponseV3 (Succeed to generate user token by one time link code.)

        ---: HttpResponse (Undocumented Response)

        ---: HttpResponse (Unexpected Content-Type Error)

        ---: HttpResponse (Unhandled Error)
        """
        pre_processed_response, error = self.pre_process_response(
            code=code, content_type=content_type, content=content
        )
        if error is not None:
            return None, None if error.is_no_content() else error
        code, content_type, content = pre_processed_response

        if code == 200:
            return OauthmodelTokenResponseV3.create_from_dict(content), None

        return self.handle_undocumented_response(
            code=code, content_type=content_type, content=content
        )

    # endregion response methods

    # region static methods

    @classmethod
    def create(
        cls,
        client_id: str,
        one_time_link_code: str,
        additional_data: Optional[str] = None,
        is_transient: Optional[bool] = None,
        **kwargs,
    ) -> RequestTokenByOneTimeLinkCodeResponseV3:
        instance = cls()
        instance.client_id = client_id
        instance.one_time_link_code = one_time_link_code
        if additional_data is not None:
            instance.additional_data = additional_data
        if is_transient is not None:
            instance.is_transient = is_transient
        if x_flight_id := kwargs.get("x_flight_id", None):
            instance.x_flight_id = x_flight_id
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> RequestTokenByOneTimeLinkCodeResponseV3:
        instance = cls()
        if "additionalData" in dict_ and dict_["additionalData"] is not None:
            instance.additional_data = str(dict_["additionalData"])
        elif include_empty:
            instance.additional_data = ""
        if "isTransient" in dict_ and dict_["isTransient"] is not None:
            instance.is_transient = bool(dict_["isTransient"])
        elif include_empty:
            instance.is_transient = False
        if "client_id" in dict_ and dict_["client_id"] is not None:
            instance.client_id = str(dict_["client_id"])
        elif include_empty:
            instance.client_id = ""
        if "oneTimeLinkCode" in dict_ and dict_["oneTimeLinkCode"] is not None:
            instance.one_time_link_code = str(dict_["oneTimeLinkCode"])
        elif include_empty:
            instance.one_time_link_code = ""
        return instance

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "additionalData": "additional_data",
            "isTransient": "is_transient",
            "client_id": "client_id",
            "oneTimeLinkCode": "one_time_link_code",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "additionalData": False,
            "isTransient": False,
            "client_id": True,
            "oneTimeLinkCode": True,
        }

    # endregion static methods



# Copyright (c) 2025 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.

from dataclasses import dataclass
from typing import Optional, Tuple

from ...core import (
    SDK,
    AccelByteSDK,
    HttpResponse,
    Operation,
    ProtoHttpRequest,
    RequestPreprocessor,
)

from ._refresher import (
    TokenRefresher,
    TokenRefresherOptions,
)

DEFAULT_ON_DEMAND_KEY: str = "on_demand_refresh"


@dataclass
class OnDemandOptions(TokenRefresherOptions):
    pass


class OnDemandLogin:
    def __init__(
        self,
        refresher: TokenRefresher,
        /,
        sdk: Optional[AccelByteSDK] = None,
        **kwargs,
    ) -> None:
        sdk = sdk or SDK

        self.refresher = refresher
        self.sdk = sdk

    def __call__(
        self,
        proto: ProtoHttpRequest,
        operation: Operation,
        sdk: AccelByteSDK,
        *args,
        **kwargs,
    ) -> Tuple[ProtoHttpRequest, Optional[HttpResponse]]:
        token_repo = sdk.get_token_repository(raise_when_none=False)
        if token_repo is None:
            return proto, HttpResponse.create_token_repo_not_found_error()

        self.run()

        return proto, None

    def run(self):
        self.refresher.refresh(sdk=self.sdk)


assert issubclass(OnDemandLogin, RequestPreprocessor)


def enable_on_demand_refresh(
    sdk: AccelByteSDK,
    options: Optional[OnDemandOptions] = None,
    key: str = DEFAULT_ON_DEMAND_KEY,
    **kwargs,
) -> None:
    options = options or OnDemandOptions()
    refresher = TokenRefresher(options=options)
    on_demand = OnDemandLogin(refresher, sdk=sdk)

    disable_on_demand_refresh(sdk=sdk, key=key, **kwargs)

    sdk.request_preprocessors[key] = on_demand


def disable_on_demand_refresh(
    sdk: AccelByteSDK, key: str = DEFAULT_ON_DEMAND_KEY, **kwargs
) -> None:
    sdk.request_preprocessors.pop(key, None)


__all__ = [
    "OnDemandLogin",
    "OnDemandOptions",
    "enable_on_demand_refresh",
    "disable_on_demand_refresh",
]



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

"""Auto-generated package that contains models used by the AccelByte Gaming Services Gdpr Service."""

__version__ = "2.24.0"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from .public_cancel_my_person_702fb0 import PublicCancelMyPersonalDataRequest
from .public_generate_my_head_3818b6 import PublicGenerateMyHeadlessPersonalDataURL
from .public_get_my_personal__1068bc import PublicGetMyPersonalDataRequests
from .public_submit_my_headle_42af28 import PublicSubmitMyHeadlessPersonalDataRequest
from .public_submit_my_person_811b35 import PublicSubmitMyPersonalDataRequest



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Iam Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.accountcommon_group_and_role_mapping_for_patch import (
    AccountcommonGroupAndRoleMappingForPatch,
)


class AccountcommonSsoConfigPatchReq(Model):
    """Accountcommon sso config patch req (accountcommon.SsoConfigPatchReq)

    Properties:
        google_key: (googleKey) OPTIONAL Dict[str, Any]

        group_configs: (groupConfigs) OPTIONAL List[AccountcommonGroupAndRoleMappingForPatch]
    """

    # region fields

    google_key: Dict[str, Any]  # OPTIONAL
    group_configs: List[AccountcommonGroupAndRoleMappingForPatch]  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_google_key(self, value: Dict[str, Any]) -> AccountcommonSsoConfigPatchReq:
        self.google_key = value
        return self

    def with_group_configs(
        self, value: List[AccountcommonGroupAndRoleMappingForPatch]
    ) -> AccountcommonSsoConfigPatchReq:
        self.group_configs = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "google_key"):
            result["googleKey"] = {str(k0): v0 for k0, v0 in self.google_key.items()}
        elif include_empty:
            result["googleKey"] = {}
        if hasattr(self, "group_configs"):
            result["groupConfigs"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.group_configs
            ]
        elif include_empty:
            result["groupConfigs"] = []
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        google_key: Optional[Dict[str, Any]] = None,
        group_configs: Optional[List[AccountcommonGroupAndRoleMappingForPatch]] = None,
        **kwargs,
    ) -> AccountcommonSsoConfigPatchReq:
        instance = cls()
        if google_key is not None:
            instance.google_key = google_key
        if group_configs is not None:
            instance.group_configs = group_configs
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> AccountcommonSsoConfigPatchReq:
        instance = cls()
        if not dict_:
            return instance
        if "googleKey" in dict_ and dict_["googleKey"] is not None:
            instance.google_key = {str(k0): v0 for k0, v0 in dict_["googleKey"].items()}
        elif include_empty:
            instance.google_key = {}
        if "groupConfigs" in dict_ and dict_["groupConfigs"] is not None:
            instance.group_configs = [
                AccountcommonGroupAndRoleMappingForPatch.create_from_dict(
                    i0, include_empty=include_empty
                )
                for i0 in dict_["groupConfigs"]
            ]
        elif include_empty:
            instance.group_configs = []
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, AccountcommonSsoConfigPatchReq]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[AccountcommonSsoConfigPatchReq]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        AccountcommonSsoConfigPatchReq,
        List[AccountcommonSsoConfigPatchReq],
        Dict[Any, AccountcommonSsoConfigPatchReq],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "googleKey": "google_key",
            "groupConfigs": "group_configs",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "googleKey": False,
            "groupConfigs": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import deprecated
from ....core import same_doc_as

from ..models import AccountcommonBanReasons
from ..models import AccountcommonBanReasonsV3
from ..models import AccountcommonBans
from ..models import AccountcommonBansV3
from ..models import ModelBulkBanCreateRequestV3
from ..models import ModelBulkUnbanCreateRequestV3
from ..models import ModelGetUserBanV3Response
from ..models import ModelListBulkUserBanResponseV3
from ..models import RestErrorResponse

from ..operations.bans import AdminBanUserBulkV3
from ..operations.bans import AdminGetBannedUsersV3
from ..operations.bans import AdminGetBansTypeV3
from ..operations.bans import AdminGetBansTypeWithNamespaceV3
from ..operations.bans import AdminGetListBanReasonV3
from ..operations.bans import AdminUnbanUserBulkV3
from ..operations.bans import GetBansType
from ..operations.bans import GetListBanReason


@same_doc_as(AdminBanUserBulkV3)
def admin_ban_user_bulk_v3(
    body: ModelBulkBanCreateRequestV3,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Bulk ban user (AdminBanUserBulkV3)

    Bulk ban user with specific type of ban. Ban types and reason can be queried. The maximum limit value is 100 userIDs

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/bans/users

        method: POST

        tags: ["Bans"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelBulkBanCreateRequestV3 in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - ModelListBulkUserBanResponseV3 (Created)

        400: Bad Request - RestErrorResponse (20002: validation error | 20019: unable to parse request body)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - RestErrorResponse (20008: user not found | 10139: platform account not found | 10158: ban not found)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminBanUserBulkV3.create(
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminBanUserBulkV3)
async def admin_ban_user_bulk_v3_async(
    body: ModelBulkBanCreateRequestV3,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Bulk ban user (AdminBanUserBulkV3)

    Bulk ban user with specific type of ban. Ban types and reason can be queried. The maximum limit value is 100 userIDs

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/bans/users

        method: POST

        tags: ["Bans"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelBulkBanCreateRequestV3 in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - ModelListBulkUserBanResponseV3 (Created)

        400: Bad Request - RestErrorResponse (20002: validation error | 20019: unable to parse request body)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - RestErrorResponse (20008: user not found | 10139: platform account not found | 10158: ban not found)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminBanUserBulkV3.create(
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminGetBannedUsersV3)
def admin_get_banned_users_v3(
    active_only: Optional[bool] = None,
    ban_type: Optional[str] = None,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get list of user filtered by ban types (AdminGetBannedUsersV3)

    Retrieves a paginated list of users who have active or history bans, filtered by ban type.

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/bans/users

        method: GET

        tags: ["Bans"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        active_only: (activeOnly) OPTIONAL bool in query

        ban_type: (banType) OPTIONAL str in query

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

    Responses:
        200: OK - ModelGetUserBanV3Response (OK)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminGetBannedUsersV3.create(
        active_only=active_only,
        ban_type=ban_type,
        limit=limit,
        offset=offset,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminGetBannedUsersV3)
async def admin_get_banned_users_v3_async(
    active_only: Optional[bool] = None,
    ban_type: Optional[str] = None,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get list of user filtered by ban types (AdminGetBannedUsersV3)

    Retrieves a paginated list of users who have active or history bans, filtered by ban type.

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/bans/users

        method: GET

        tags: ["Bans"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        active_only: (activeOnly) OPTIONAL bool in query

        ban_type: (banType) OPTIONAL str in query

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

    Responses:
        200: OK - ModelGetUserBanV3Response (OK)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminGetBannedUsersV3.create(
        active_only=active_only,
        ban_type=ban_type,
        limit=limit,
        offset=offset,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminGetBansTypeV3)
def admin_get_bans_type_v3(
    x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    """Get list of ban types (AdminGetBansTypeV3)

    Ban type is the code
    available for ban assignment. It is applicable globally for any namespace.

    Properties:
        url: /iam/v3/admin/bans

        method: GET

        tags: ["Bans"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

    Responses:
        200: OK - AccountcommonBansV3 (OK)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    request = AdminGetBansTypeV3.create()
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminGetBansTypeV3)
async def admin_get_bans_type_v3_async(
    x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    """Get list of ban types (AdminGetBansTypeV3)

    Ban type is the code
    available for ban assignment. It is applicable globally for any namespace.

    Properties:
        url: /iam/v3/admin/bans

        method: GET

        tags: ["Bans"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

    Responses:
        200: OK - AccountcommonBansV3 (OK)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    request = AdminGetBansTypeV3.create()
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminGetBansTypeWithNamespaceV3)
def admin_get_bans_type_with_namespace_v3(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get list of ban types (AdminGetBansTypeWithNamespaceV3)

    Ban type is the code
    available for ban assignment. It is applicable globally for any namespace.

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/bantypes

        method: GET

        tags: ["Bans"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - AccountcommonBansV3 (OK)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminGetBansTypeWithNamespaceV3.create(
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminGetBansTypeWithNamespaceV3)
async def admin_get_bans_type_with_namespace_v3_async(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get list of ban types (AdminGetBansTypeWithNamespaceV3)

    Ban type is the code
    available for ban assignment. It is applicable globally for any namespace.

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/bantypes

        method: GET

        tags: ["Bans"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - AccountcommonBansV3 (OK)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminGetBansTypeWithNamespaceV3.create(
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminGetListBanReasonV3)
def admin_get_list_ban_reason_v3(
    x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    """Get list of ban reasons (AdminGetListBanReasonV3)

    Ban reasons is the code available to justify ban assignment. It is applicable globally for any namespace.

    Properties:
        url: /iam/v3/admin/bans/reasons

        method: GET

        tags: ["Bans"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

    Responses:
        200: OK - AccountcommonBanReasonsV3 (OK)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    request = AdminGetListBanReasonV3.create()
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminGetListBanReasonV3)
async def admin_get_list_ban_reason_v3_async(
    x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    """Get list of ban reasons (AdminGetListBanReasonV3)

    Ban reasons is the code available to justify ban assignment. It is applicable globally for any namespace.

    Properties:
        url: /iam/v3/admin/bans/reasons

        method: GET

        tags: ["Bans"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

    Responses:
        200: OK - AccountcommonBanReasonsV3 (OK)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    request = AdminGetListBanReasonV3.create()
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminUnbanUserBulkV3)
def admin_unban_user_bulk_v3(
    body: ModelBulkUnbanCreateRequestV3,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Bulk unban user (AdminUnbanUserBulkV3)

    disable bulk ban user. The maximum limit value is 100

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/bans/users/disabled

        method: PATCH

        tags: ["Bans"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelBulkUnbanCreateRequestV3 in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - ModelListBulkUserBanResponseV3 (Created)

        400: Bad Request - RestErrorResponse (20002: validation error | 20019: unable to parse request body)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - RestErrorResponse (20008: user not found | 10139: platform account not found | 10158: ban not found)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminUnbanUserBulkV3.create(
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminUnbanUserBulkV3)
async def admin_unban_user_bulk_v3_async(
    body: ModelBulkUnbanCreateRequestV3,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Bulk unban user (AdminUnbanUserBulkV3)

    disable bulk ban user. The maximum limit value is 100

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/bans/users/disabled

        method: PATCH

        tags: ["Bans"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelBulkUnbanCreateRequestV3 in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - ModelListBulkUserBanResponseV3 (Created)

        400: Bad Request - RestErrorResponse (20002: validation error | 20019: unable to parse request body)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - RestErrorResponse (20008: user not found | 10139: platform account not found | 10158: ban not found)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminUnbanUserBulkV3.create(
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@deprecated
@same_doc_as(GetBansType)
def get_bans_type(x_additional_headers: Optional[Dict[str, str]] = None, **kwargs):
    """Get list of ban types (GetBansType)

    **This endpoint is deprecated.**
    ### Endpoint migration guide
    - **Substitute endpoint: _/iam/v3/admin/bans [GET]_**

    Properties:
        url: /iam/bans

        method: GET

        tags: ["Bans"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

    Responses:
        200: OK - AccountcommonBans (OK)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)
    """
    request = GetBansType.create()
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(GetBansType)
async def get_bans_type_async(
    x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    """Get list of ban types (GetBansType)

    **This endpoint is deprecated.**
    ### Endpoint migration guide
    - **Substitute endpoint: _/iam/v3/admin/bans [GET]_**

    Properties:
        url: /iam/bans

        method: GET

        tags: ["Bans"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

    Responses:
        200: OK - AccountcommonBans (OK)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)
    """
    request = GetBansType.create()
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@deprecated
@same_doc_as(GetListBanReason)
def get_list_ban_reason(
    x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    """Get list of ban reasons (GetListBanReason)

    **This endpoint is deprecated.**
    ### Endpoint migration guide
    - **Substitute endpoint: _/iam/v3/admin/bans/reasons [GET]_**

    Properties:
        url: /iam/bans/reasons

        method: GET

        tags: ["Bans"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

    Responses:
        200: OK - AccountcommonBanReasons (OK)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)
    """
    request = GetListBanReason.create()
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(GetListBanReason)
async def get_list_ban_reason_async(
    x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    """Get list of ban reasons (GetListBanReason)

    **This endpoint is deprecated.**
    ### Endpoint migration guide
    - **Substitute endpoint: _/iam/v3/admin/bans/reasons [GET]_**

    Properties:
        url: /iam/bans/reasons

        method: GET

        tags: ["Bans"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

    Responses:
        200: OK - AccountcommonBanReasons (OK)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)
    """
    request = GetListBanReason.create()
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Iam Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.accountcommon_query_cursor import AccountcommonQueryCursor


class ModelCursorGetUserRequest(Model):
    """Model cursor get user request (model.CursorGetUserRequest)

    Properties:
        limit: (limit) REQUIRED int

        cursor: (cursor) OPTIONAL AccountcommonQueryCursor

        fields: (fields) OPTIONAL List[str]
    """

    # region fields

    limit: int  # REQUIRED
    cursor: AccountcommonQueryCursor  # OPTIONAL
    fields: List[str]  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_limit(self, value: int) -> ModelCursorGetUserRequest:
        self.limit = value
        return self

    def with_cursor(self, value: AccountcommonQueryCursor) -> ModelCursorGetUserRequest:
        self.cursor = value
        return self

    def with_fields(self, value: List[str]) -> ModelCursorGetUserRequest:
        self.fields = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "limit"):
            result["limit"] = int(self.limit)
        elif include_empty:
            result["limit"] = 0
        if hasattr(self, "cursor"):
            result["cursor"] = self.cursor.to_dict(include_empty=include_empty)
        elif include_empty:
            result["cursor"] = AccountcommonQueryCursor()
        if hasattr(self, "fields"):
            result["fields"] = [str(i0) for i0 in self.fields]
        elif include_empty:
            result["fields"] = []
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        limit: int,
        cursor: Optional[AccountcommonQueryCursor] = None,
        fields: Optional[List[str]] = None,
        **kwargs,
    ) -> ModelCursorGetUserRequest:
        instance = cls()
        instance.limit = limit
        if cursor is not None:
            instance.cursor = cursor
        if fields is not None:
            instance.fields = fields
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ModelCursorGetUserRequest:
        instance = cls()
        if not dict_:
            return instance
        if "limit" in dict_ and dict_["limit"] is not None:
            instance.limit = int(dict_["limit"])
        elif include_empty:
            instance.limit = 0
        if "cursor" in dict_ and dict_["cursor"] is not None:
            instance.cursor = AccountcommonQueryCursor.create_from_dict(
                dict_["cursor"], include_empty=include_empty
            )
        elif include_empty:
            instance.cursor = AccountcommonQueryCursor()
        if "fields" in dict_ and dict_["fields"] is not None:
            instance.fields = [str(i0) for i0 in dict_["fields"]]
        elif include_empty:
            instance.fields = []
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ModelCursorGetUserRequest]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ModelCursorGetUserRequest]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ModelCursorGetUserRequest,
        List[ModelCursorGetUserRequest],
        Dict[Any, ModelCursorGetUserRequest],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "limit": "limit",
            "cursor": "cursor",
            "fields": "fields",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "limit": True,
            "cursor": False,
            "fields": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

# AccelByte Gaming Services Platform Service

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from .....core import Operation
from .....core import HeaderStr
from .....core import HttpResponse

from ...models import ErrorEntity
from ...models import FullItemInfo
from ...models import ItemCreate
from ...models import ValidationErrorEntity


class CreateItem(Operation):
    """Create an Item (createItem)

    This API is used to create an item. APP item only can created in publisher namespace.

    An item create example:


        {

           "itemId": "b478be9ad42f4d0f8ba49ef3ef605a83" (optional, UUID V4 without hyphen format)

           "categoryPath": "/games",

           "localizations": {

               "en": {

                   "title":"required",

                   "description":"optional",

                   "longDescription":"optional",

                   "localExt": {

                          "properties":[

                                           {

                                               "key1":"value1",

                                               "key2":"value2"

                                           }

                          ],

                          "functions":[

                                           {

                                               "key1":"value1",

                                               "key2":"value2"

                                           }

                         ]

                   }

                }

           },

           "images": [

             {

                   "as":"optional, image for",

                   "caption":"optional",

                   "height":10,

                   "width":10,

                   "imageUrl":"http://img-url-required",

                   "smallImageUrl":"http://small-img-url-required"

             }

           ],

           "thumbnailUrl": "optional, thumbnail url",

           "status": "ACTIVE",

           "listable": true,

           "purchasable": true,

           "itemType": "APP(allowed: [APP,COINS,INGAMEITEM,CODE,BUNDLE])",

           "name": "required, also will be used as entitlement name",

           "entitlementType": "DURABLE(allowed:[DURABLE,CONSUMABLE], should be CONSUMABLE when item type is COINS)",

           "useCount": 1(optional, required if the entitlement type is consumable),

           "stackable": false,

           "appId": "optional, required if itemType is APP",

           "appType": "GAME(optional, required if itemType is APP)",

           "seasonType": "PASS(optional, required if itemType is SEASON)",

           "baseAppId": "optional, set value of game app id if you want to link to a game",

           "targetCurrencyCode": "optional, required if itemType is COINS",

           "targetNamespace": "optional, required when itemType is INGAMEITEM, the targetNamespace will only take effect when the item

           created belongs to the publisher namespace",

           "sku": "optional, commonly unique item code",

           "regionData": {

               "US(store's default region is required)": [

                 {

                    "price":10,

                    "discountPercentage": 0(integer, optional, range[0,100], discountedPrice = price  * ((100 - discountPercentage) * 0.01),

                      will use it to calculate discounted price if it is not 0),

                    "discountAmount":0(integer, optional, range[0,itemPrice], will use it to calculate discounted price if discountPercentage is 0),

                    "currencyCode":"code(required, example: USD)",

                    "currencyNamespace":"test-ns-required(allow publisher namespace if namespace is publisher namespace, allow publisher and game namespace if namespace is not publisher namespace)",

                    "trialPrice":5(required while fixedTrialCycles set, should >=0 and <= price, will same as price if not present),

                    "purchaseAt":"optional yyyy-MM-dd'T'HH:mm:ss.SSS'Z'",

                    "expireAt":"optional yyyy-MM-dd'T'HH:mm:ss.SSS'Z'",

                    "discountPurchaseAt":"optional yyyy-MM-dd'T'HH:mm:ss.SSS'Z'",

                    "discountExpireAt":"optional yyyy-MM-dd'T'HH:mm:ss.SSS'Z'"

                 }

               ]

           },

           "itemIds": [

               "itemId"

           ],

           "itemQty": {

               "itemId":1

           },

           "recurring": {

               "cycle":"MONTHLY(allowed: [WEEKLY,MONTHLY,QUARTERLY,YEARLY])",

               "fixedFreeDays":0(integer, fixed free days, 0 means not set),

               "fixedTrialCycles":0(integer, fixed trial cycles, 0 means not set, will not take effect if fixedFreeDays set),

               "graceDays":7(integer, recurring grace days, retry recurring charge within configured days if charge fail, default 7)

           },

           "tags": [

               "exampleTag24"

           ],

           "features": [

               "feature"

           ],

           "clazz": "weapon",

           "boothName": "C_campaign1",

           "displayOrder": 1000,

           "ext": {

               "properties":[

                   {

                       "key1":"value1",

                       "key2":"value2"

                   }

               ],

               "functions":[

                   {

                       "key1":"value1",

                       "key2":"value2"

                   }

               ]

           },

           "maxCountPerUser": 1(integer, optional, -1 means UNLIMITED),

           "maxCount": 1(integer, optional, -1 means UNLIMITED, unset when itemType is CODE)

        }

    Other detail info:

      * Returns : created item data



    ## Restrictions for item extension and localization extension and inventory config custom attributes


    1. Cannot use "." as the key name
    -


        { "data.2": "value" }


    2. Cannot use "$" as the prefix in key names
    -


        { "$data": "value" }

    Properties:
        url: /platform/admin/namespaces/{namespace}/items

        method: POST

        tags: ["Item"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ItemCreate in body

        namespace: (namespace) REQUIRED str in path

        store_id: (storeId) REQUIRED str in query

    Responses:
        201: Created - FullItemInfo (successful operation)

        400: Bad Request - ErrorEntity (20024: unsupported operation | 30301: Unsupported Item Type [{itemType}] for box item [{itemId}] with expiration | 30322: Bundle item [{itemId}] can't be bundled | 30325: Code item [{itemId}] can't be bundled | 30326: Subscription item [{itemId}] can't be bundled | 30329: Invalid bundled item [{itemId}] quantity | 30021: Default language [{language}] required | 30321: Invalid item discount amount | 30022: Default region [{region}] is required | 30323: Target namespace is required | 30327: Invalid item trial price | 30330: Invalid item region price currency namespace [{namespace}] | 30332: Invalid option box item [{itemId}] quantity | 30333: Item [{itemId}] item type [{itemType}] can't be bundled into option box | 30334: Option box item [{itemId}] can't be bundled | 30337: Invalid loot box item [{itemId}] quantity | 30338: Item [{itemId}] item type [{itemType}] can't be bundled into loot box | 30339: Loot box item [{itemId}] can't be bundled)

        404: Not Found - ErrorEntity (30241: Category [{categoryPath}] does not exist in namespace [{namespace}] | 36141: Currency [{currencyCode}] does not exist in namespace [{namespace}] | 30141: Store [{storeId}] does not exist in namespace [{namespace}])

        409: Conflict - ErrorEntity (30173: Published store can't modify content | 30175: Duplicated currencyCode [{currencyCode}] in Region [{region}] | 30373: ItemType [{itemType}] is not allowed in namespace [{namespace}] | 30376: Publisher namespace don’t allow sellback item | 30377: This item type [{itemType}] don’t allow sellback | 30378: Sale price don’t allow real currency [{currencyCode}] | 30380: Box item [{itemId}] duration and end date can’t be set at the same time | 30381: Currency [{currency}] is not set for bundle Item [{itemId}] in region [{region}] | 30383: Item app id [{appId}] already exists in namespace [{namespace}] item [{itemId}])

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)
    """

    # region fields

    _url: str = "/platform/admin/namespaces/{namespace}/items"
    _method: str = "POST"
    _consumes: List[str] = ["application/json"]
    _produces: List[str] = ["application/json"]
    _securities: List[List[str]] = [["BEARER_AUTH"]]
    _location_query: str = None

    body: ItemCreate  # REQUIRED in [body]
    namespace: str  # REQUIRED in [path]
    store_id: str  # REQUIRED in [query]

    # endregion fields

    # region properties

    @property
    def url(self) -> str:
        return self._url

    @property
    def method(self) -> str:
        return self._method

    @property
    def consumes(self) -> List[str]:
        return self._consumes

    @property
    def produces(self) -> List[str]:
        return self._produces

    @property
    def securities(self) -> List[List[str]]:
        return self._securities

    @property
    def location_query(self) -> str:
        return self._location_query

    # endregion properties

    # region get methods

    # endregion get methods

    # region get_x_params methods

    def get_all_params(self) -> dict:
        return {
            "body": self.get_body_params(),
            "path": self.get_path_params(),
            "query": self.get_query_params(),
        }

    def get_body_params(self) -> Any:
        if not hasattr(self, "body") or self.body is None:
            return None
        return self.body.to_dict()

    def get_path_params(self) -> dict:
        result = {}
        if hasattr(self, "namespace"):
            result["namespace"] = self.namespace
        return result

    def get_query_params(self) -> dict:
        result = {}
        if hasattr(self, "store_id"):
            result["storeId"] = self.store_id
        return result

    # endregion get_x_params methods

    # region is/has methods

    # endregion is/has methods

    # region with_x methods

    def with_body(self, value: ItemCreate) -> CreateItem:
        self.body = value
        return self

    def with_namespace(self, value: str) -> CreateItem:
        self.namespace = value
        return self

    def with_store_id(self, value: str) -> CreateItem:
        self.store_id = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "body") and self.body:
            result["body"] = self.body.to_dict(include_empty=include_empty)
        elif include_empty:
            result["body"] = ItemCreate()
        if hasattr(self, "namespace") and self.namespace:
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "store_id") and self.store_id:
            result["storeId"] = str(self.store_id)
        elif include_empty:
            result["storeId"] = ""
        return result

    # endregion to methods

    # region response methods

    # noinspection PyMethodMayBeStatic
    def parse_response(
        self, code: int, content_type: str, content: Any
    ) -> Tuple[
        Union[None, FullItemInfo],
        Union[None, ErrorEntity, HttpResponse, ValidationErrorEntity],
    ]:
        """Parse the given response.

        201: Created - FullItemInfo (successful operation)

        400: Bad Request - ErrorEntity (20024: unsupported operation | 30301: Unsupported Item Type [{itemType}] for box item [{itemId}] with expiration | 30322: Bundle item [{itemId}] can't be bundled | 30325: Code item [{itemId}] can't be bundled | 30326: Subscription item [{itemId}] can't be bundled | 30329: Invalid bundled item [{itemId}] quantity | 30021: Default language [{language}] required | 30321: Invalid item discount amount | 30022: Default region [{region}] is required | 30323: Target namespace is required | 30327: Invalid item trial price | 30330: Invalid item region price currency namespace [{namespace}] | 30332: Invalid option box item [{itemId}] quantity | 30333: Item [{itemId}] item type [{itemType}] can't be bundled into option box | 30334: Option box item [{itemId}] can't be bundled | 30337: Invalid loot box item [{itemId}] quantity | 30338: Item [{itemId}] item type [{itemType}] can't be bundled into loot box | 30339: Loot box item [{itemId}] can't be bundled)

        404: Not Found - ErrorEntity (30241: Category [{categoryPath}] does not exist in namespace [{namespace}] | 36141: Currency [{currencyCode}] does not exist in namespace [{namespace}] | 30141: Store [{storeId}] does not exist in namespace [{namespace}])

        409: Conflict - ErrorEntity (30173: Published store can't modify content | 30175: Duplicated currencyCode [{currencyCode}] in Region [{region}] | 30373: ItemType [{itemType}] is not allowed in namespace [{namespace}] | 30376: Publisher namespace don’t allow sellback item | 30377: This item type [{itemType}] don’t allow sellback | 30378: Sale price don’t allow real currency [{currencyCode}] | 30380: Box item [{itemId}] duration and end date can’t be set at the same time | 30381: Currency [{currency}] is not set for bundle Item [{itemId}] in region [{region}] | 30383: Item app id [{appId}] already exists in namespace [{namespace}] item [{itemId}])

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)

        ---: HttpResponse (Undocumented Response)

        ---: HttpResponse (Unexpected Content-Type Error)

        ---: HttpResponse (Unhandled Error)
        """
        pre_processed_response, error = self.pre_process_response(
            code=code, content_type=content_type, content=content
        )
        if error is not None:
            return None, None if error.is_no_content() else error
        code, content_type, content = pre_processed_response

        if code == 201:
            return FullItemInfo.create_from_dict(content), None
        if code == 400:
            return None, ErrorEntity.create_from_dict(content)
        if code == 404:
            return None, ErrorEntity.create_from_dict(content)
        if code == 409:
            return None, ErrorEntity.create_from_dict(content)
        if code == 422:
            return None, ValidationErrorEntity.create_from_dict(content)

        return self.handle_undocumented_response(
            code=code, content_type=content_type, content=content
        )

    # endregion response methods

    # region static methods

    @classmethod
    def create(
        cls, body: ItemCreate, namespace: str, store_id: str, **kwargs
    ) -> CreateItem:
        instance = cls()
        instance.body = body
        instance.namespace = namespace
        instance.store_id = store_id
        if x_flight_id := kwargs.get("x_flight_id", None):
            instance.x_flight_id = x_flight_id
        return instance

    @classmethod
    def create_from_dict(cls, dict_: dict, include_empty: bool = False) -> CreateItem:
        instance = cls()
        if "body" in dict_ and dict_["body"] is not None:
            instance.body = ItemCreate.create_from_dict(
                dict_["body"], include_empty=include_empty
            )
        elif include_empty:
            instance.body = ItemCreate()
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "storeId" in dict_ and dict_["storeId"] is not None:
            instance.store_id = str(dict_["storeId"])
        elif include_empty:
            instance.store_id = ""
        return instance

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "body": "body",
            "namespace": "namespace",
            "storeId": "store_id",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "body": True,
            "namespace": True,
            "storeId": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# Custom Service Manager

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class DomainKeyValueClusterConfig(Model):
    """Domain key value cluster config (domain.KeyValueClusterConfig)

    Properties:
        max_cluster_per_studio_private_cloud: (max_cluster_per_studio_private_cloud) REQUIRED int

        max_cluster_per_studio_shared_cloud: (max_cluster_per_studio_shared_cloud) REQUIRED int

        max_integration_per_cluster: (max_integration_per_cluster) REQUIRED int
    """

    # region fields

    max_cluster_per_studio_private_cloud: int  # REQUIRED
    max_cluster_per_studio_shared_cloud: int  # REQUIRED
    max_integration_per_cluster: int  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_max_cluster_per_studio_private_cloud(
        self, value: int
    ) -> DomainKeyValueClusterConfig:
        self.max_cluster_per_studio_private_cloud = value
        return self

    def with_max_cluster_per_studio_shared_cloud(
        self, value: int
    ) -> DomainKeyValueClusterConfig:
        self.max_cluster_per_studio_shared_cloud = value
        return self

    def with_max_integration_per_cluster(
        self, value: int
    ) -> DomainKeyValueClusterConfig:
        self.max_integration_per_cluster = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "max_cluster_per_studio_private_cloud"):
            result["max_cluster_per_studio_private_cloud"] = int(
                self.max_cluster_per_studio_private_cloud
            )
        elif include_empty:
            result["max_cluster_per_studio_private_cloud"] = 0
        if hasattr(self, "max_cluster_per_studio_shared_cloud"):
            result["max_cluster_per_studio_shared_cloud"] = int(
                self.max_cluster_per_studio_shared_cloud
            )
        elif include_empty:
            result["max_cluster_per_studio_shared_cloud"] = 0
        if hasattr(self, "max_integration_per_cluster"):
            result["max_integration_per_cluster"] = int(
                self.max_integration_per_cluster
            )
        elif include_empty:
            result["max_integration_per_cluster"] = 0
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        max_cluster_per_studio_private_cloud: int,
        max_cluster_per_studio_shared_cloud: int,
        max_integration_per_cluster: int,
        **kwargs,
    ) -> DomainKeyValueClusterConfig:
        instance = cls()
        instance.max_cluster_per_studio_private_cloud = (
            max_cluster_per_studio_private_cloud
        )
        instance.max_cluster_per_studio_shared_cloud = (
            max_cluster_per_studio_shared_cloud
        )
        instance.max_integration_per_cluster = max_integration_per_cluster
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> DomainKeyValueClusterConfig:
        instance = cls()
        if not dict_:
            return instance
        if (
            "max_cluster_per_studio_private_cloud" in dict_
            and dict_["max_cluster_per_studio_private_cloud"] is not None
        ):
            instance.max_cluster_per_studio_private_cloud = int(
                dict_["max_cluster_per_studio_private_cloud"]
            )
        elif include_empty:
            instance.max_cluster_per_studio_private_cloud = 0
        if (
            "max_cluster_per_studio_shared_cloud" in dict_
            and dict_["max_cluster_per_studio_shared_cloud"] is not None
        ):
            instance.max_cluster_per_studio_shared_cloud = int(
                dict_["max_cluster_per_studio_shared_cloud"]
            )
        elif include_empty:
            instance.max_cluster_per_studio_shared_cloud = 0
        if (
            "max_integration_per_cluster" in dict_
            and dict_["max_integration_per_cluster"] is not None
        ):
            instance.max_integration_per_cluster = int(
                dict_["max_integration_per_cluster"]
            )
        elif include_empty:
            instance.max_integration_per_cluster = 0
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, DomainKeyValueClusterConfig]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[DomainKeyValueClusterConfig]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        DomainKeyValueClusterConfig,
        List[DomainKeyValueClusterConfig],
        Dict[Any, DomainKeyValueClusterConfig],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "max_cluster_per_studio_private_cloud": "max_cluster_per_studio_private_cloud",
            "max_cluster_per_studio_shared_cloud": "max_cluster_per_studio_shared_cloud",
            "max_integration_per_cluster": "max_integration_per_cluster",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "max_cluster_per_studio_private_cloud": True,
            "max_cluster_per_studio_shared_cloud": True,
            "max_integration_per_cluster": True,
        }

    # endregion static methods



# Copyright (c) 2025 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.

from ._login_user import login_user_internal as login
from ._login_user import login_user_async_internal as login_async
from ._login_client import login_client_internal as login_client
from ._login_client import login_client_async_internal as login_client_async
from ._login_platform import login_platform_internal as login_platform
from ._login_platform import login_platform_async_internal as login_platform_async
from ._login_user import login_user_internal as login_user
from ._login_user import login_user_async_internal as login_user_async
from ._login_refresh import login_refresh_internal as refresh_login
from ._login_refresh import login_refresh_async_internal as refresh_login_async
from ._login_refresh import try_refresh_login_internal as try_refresh_login
from ._login_refresh import try_refresh_login_async_internal as try_refresh_login_async
from ._logout import logout_internal as logout
from ._logout import logout_async_internal as logout_async

from ._login_const import DEFAULT_AUTO_REFRESH
from ._login_const import DEFAULT_REFRESH_RATE

from ._jwt import parse_access_token

from ._utils import convert_bearer_auth_token_to_oauth_token_dict

from ._login_background import (
    BackgroundOptions,
    enable_background_refresh,
    disable_background_refresh,
)
from ._login_on_demand import (
    OnDemandOptions,
    enable_on_demand_refresh,
    disable_on_demand_refresh,
)

__all__ = [
    # login
    "login",
    "login_async",
    "login_client",
    "login_client_async",
    "login_platform",
    "login_platform_async",
    "login_user",
    "login_user_async",
    "refresh_login",
    "refresh_login_async",
    "try_refresh_login",
    "try_refresh_login_async",
    "logout",
    "logout_async",
    # const
    "DEFAULT_AUTO_REFRESH",
    "DEFAULT_REFRESH_RATE",
    # jwt
    "parse_access_token",
    # utils
    "convert_bearer_auth_token_to_oauth_token_dict",
    # background
    "BackgroundOptions",
    "enable_background_refresh",
    "disable_background_refresh",
    # on-demand
    "OnDemandOptions",
    "enable_on_demand_refresh",
    "disable_on_demand_refresh",
]
